<?php

declare(strict_types=1);

namespace Monsun\Bugtest\Tests\Unit\Controller;

use PHPUnit\Framework\MockObject\MockObject;
use TYPO3\TestingFramework\Core\AccessibleObjectInterface;
use TYPO3\TestingFramework\Core\Unit\UnitTestCase;
use TYPO3Fluid\Fluid\View\ViewInterface;

/**
 * Test case
 *
 * @author Dr. Dieter Porth <porth@monun-media.com>
 */
class TestmodelControllerTest extends UnitTestCase
{
    /**
     * @var \Monsun\Bugtest\Controller\TestmodelController|MockObject|AccessibleObjectInterface
     */
    protected $subject;

    protected function setUp(): void
    {
        parent::setUp();
        $this->subject = $this->getMockBuilder($this->buildAccessibleProxy(\Monsun\Bugtest\Controller\TestmodelController::class))
            ->onlyMethods(['redirect', 'forward', 'addFlashMessage'])
            ->disableOriginalConstructor()
            ->getMock();
    }

    protected function tearDown(): void
    {
        parent::tearDown();
    }

    /**
     * @test
     */
    public function listActionFetchesAllTestmodelsFromRepositoryAndAssignsThemToView(): void
    {
        $allTestmodels = $this->getMockBuilder(\TYPO3\CMS\Extbase\Persistence\ObjectStorage::class)
            ->disableOriginalConstructor()
            ->getMock();

        $testmodelRepository = $this->getMockBuilder(\Monsun\Bugtest\Domain\Repository\TestmodelRepository::class)
            ->onlyMethods(['findAll'])
            ->disableOriginalConstructor()
            ->getMock();
        $testmodelRepository->expects(self::once())->method('findAll')->will(self::returnValue($allTestmodels));
        $this->subject->_set('testmodelRepository', $testmodelRepository);

        $view = $this->getMockBuilder(ViewInterface::class)->getMock();
        $view->expects(self::once())->method('assign')->with('testmodels', $allTestmodels);
        $this->subject->_set('view', $view);

        $this->subject->listAction();
    }

    /**
     * @test
     */
    public function createActionAddsTheGivenTestmodelToTestmodelRepository(): void
    {
        $testmodel = new \Monsun\Bugtest\Domain\Model\Testmodel();

        $testmodelRepository = $this->getMockBuilder(\Monsun\Bugtest\Domain\Repository\TestmodelRepository::class)
            ->onlyMethods(['add'])
            ->disableOriginalConstructor()
            ->getMock();

        $testmodelRepository->expects(self::once())->method('add')->with($testmodel);
        $this->subject->_set('testmodelRepository', $testmodelRepository);

        $this->subject->createAction($testmodel);
    }

    /**
     * @test
     */
    public function editActionAssignsTheGivenTestmodelToView(): void
    {
        $testmodel = new \Monsun\Bugtest\Domain\Model\Testmodel();

        $view = $this->getMockBuilder(ViewInterface::class)->getMock();
        $this->subject->_set('view', $view);
        $view->expects(self::once())->method('assign')->with('testmodel', $testmodel);

        $this->subject->editAction($testmodel);
    }

    /**
     * @test
     */
    public function updateActionUpdatesTheGivenTestmodelInTestmodelRepository(): void
    {
        $testmodel = new \Monsun\Bugtest\Domain\Model\Testmodel();

        $testmodelRepository = $this->getMockBuilder(\Monsun\Bugtest\Domain\Repository\TestmodelRepository::class)
            ->onlyMethods(['update'])
            ->disableOriginalConstructor()
            ->getMock();

        $testmodelRepository->expects(self::once())->method('update')->with($testmodel);
        $this->subject->_set('testmodelRepository', $testmodelRepository);

        $this->subject->updateAction($testmodel);
    }

    /**
     * @test
     */
    public function editActionAssignsTheGivenTestmodelToView(): void
    {
        $testmodel = new \Monsun\Bugtest\Domain\Model\Testmodel();

        $view = $this->getMockBuilder(ViewInterface::class)->getMock();
        $this->subject->_set('view', $view);
        $view->expects(self::once())->method('assign')->with('testmodel', $testmodel);

        $this->subject->editAction($testmodel);
    }

}
